//*          10MHz LED Frequency Counter              *
// This code was written for the free c compiler that comes with MPLAB.
// The 7 segments are at RB0 to RB5,RA3. 
// Each digit is stored in its reg. and converted to 7 segments by preset
// values in digit array.
// The 8 digits are driven from 5 output by system of 2 out of 5. 
// TMR1 osc at 32KHz and gives the 2 seconds time base.
// The MCU works on the 4 MHz internal osc.
// The display is off for the period of counting.
//
// This code is free to use, for more info   http://www.moty22.co.uk       
   
#include <htc.h>

#define _XTAL_FREQ 4000000
//#define MHz RA7	//MHz LED

//prototypes
void main(void);
void display(void);

__CONFIG(LVPDIS & MCLRDIS & WDTDIS & UNPROTECT  & INTIO & UNPROTECT);	// 

bit g;
unsigned char d1,d2,d3,d4,d5,d6,d7,d8;
			
void main(void){
	unsigned long total;
	unsigned int freq2;
	unsigned char freq1;	

	
	TRISB = 0b10000000;		 //RB 0-6 outputs
	TRISA = 0b110000;
	OPTION = 0b10100000;	//TMR0 pre=1:2
	CMCON = 0b111;		//comparator off
	T1CON = 0b1110;        //TMR1 prescale 1:1, osc=on
	
	d1=d2=d3=d4=d5=d6=d7=d8=0;
	display();	
for(;;){
	freq1 = 0;
	freq2 = 0;
	TMR1L = 0;
	TMR1H = 0;
	TMR1IF = 0;
	TMR1ON = 1;		// open gate
	TMR0 = 0;		// start count
	T0IF=0;
	
	while (!TMR1IF){		//loop for 2 second
		if (T0IF){++freq2; T0IF=0;}
	}
	
	freq1=TMR0;		//copy TRM0 and stop count
	TMR1ON = 0;		//close gate 

	total=(unsigned long)freq1 + (unsigned long)freq2 * 256;

		 //binary to decimal
		d8= total/10000000;
		d7=(total/1000000) %10; 
		d6=(total/100000) %10;
		d5=(total/10000) %10;
		d4=(total/1000) %10;
		d3=(total/100) %10;
		d2=(total/10) %10;
		d1=total %10;
		
	display();
}		
}
						
void display(void){
	unsigned char i; i=0;	
	unsigned char digit[12];
	digit[0]=0b111111;		//7 segments
	digit[1]=0b110;
	digit[2]=0b1011011;
	digit[3]=0b1001111;
	digit[4]=0b1100110;
	digit[5]=0b1101101;
	digit[6]=0b1111100;
	digit[7]=0b111;
	digit[8]=0b1111111;
	digit[9]=0b1100111;
	
	while(i < 240){			//display for 4 seconds
		PORTB=digit[d1];
		g=(digit[d1] >> 6); 	//g segmentis on RA6
		RA6=g;
		RA0=1; RA1=1;
		__delay_ms(2);
		RA0=0; RA1=0; RA6=0; 
		
		PORTB=digit[d2];
		g=(digit[d2] >> 6);
		RA6=g;
		RA2=1; RA3=1; 
		__delay_ms(2);
		RA2=0; RA3=0; RA6=0;
		
		PORTB=digit[d3];
		g=(digit[d3] >> 6);
		RA6=g;
		RA0=1; RA2=1;
		__delay_ms(2);
		RA0=0; RA2=0; RA6=0;
		
		PORTB=digit[d4];
		g=(digit[d4] >> 6);
		RA6=g;
		RA2=1; RA1=1;
		__delay_ms(2);
		RA2=0; RA1=0; RA6=0;
		
		PORTB=digit[d5];
		g=(digit[d5] >> 6);
		RA6=g;
		RA0=1; RA3=1; 
		__delay_ms(2);
		RA0=0; RA3=0; RA6=0;
		
		PORTB=digit[d6];
		g=(digit[d6] >> 6);
		RA6=g;
		RA3=1; RA1=1;
		__delay_ms(2);
		RA3=0; RA1=0; RA6=0;
		
		PORTB=digit[d7];
		g=(digit[d7] >> 6);
		RA6=g;
		RA0=1; RA7=1; 
		__delay_ms(2);
		RA0=0; RA7=0; RA6=0;
		
		PORTB=digit[d8];
		g=(digit[d8] >> 6);
		RA6=g;
		RA7=1; RA1=1;
		__delay_ms(2);
		RA7=0; RA1=0; RA6=0;		
		i++;
	}
}	
	